@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -55,28 +55,18 @@ inline HeapWord* G1Allocator::attempt_allocation(size_t min_word_size,
   uint node_index = current_node_index();
 
   HeapWord* result = mutator_alloc_region(node_index)->attempt_retained_allocation(min_word_size, desired_word_size, actual_word_size);
-  if (result != NULL) {
+  if (result != nullptr) {
     return result;
   }
 
   return mutator_alloc_region(node_index)->attempt_allocation(min_word_size, desired_word_size, actual_word_size);
 }
 
-inline HeapWord* G1Allocator::attempt_allocation_using_new_region(size_t word_size) {
-  uint node_index = current_node_index();
-  size_t temp;
-  HeapWord* result = mutator_alloc_region(node_index)->attempt_allocation_using_new_region(word_size, word_size, &temp);
-  assert(result != NULL || mutator_alloc_region(node_index)->get() == NULL,
-         "Must not have a mutator alloc region if there is no memory, but is " PTR_FORMAT,
-         p2i(mutator_alloc_region(node_index)->get()));
-  return result;
-}
-
 inline HeapWord* G1Allocator::attempt_allocation_locked(size_t word_size) {
   uint node_index = current_node_index();
   HeapWord* result = mutator_alloc_region(node_index)->attempt_allocation_locked(word_size);
 
-  assert(result != NULL || mutator_alloc_region(node_index)->get() == NULL,
+  assert(result != nullptr || mutator_alloc_region(node_index)->get() == nullptr,
          "Must not have a mutator alloc region if there is no memory, but is " PTR_FORMAT, p2i(mutator_alloc_region(node_index)->get()));
   return result;
 }
@@ -89,8 +79,8 @@ inline HeapWord* G1Allocator::attempt_allocation_force(size_t word_size) {
 inline PLAB* G1PLABAllocator::alloc_buffer(G1HeapRegionAttr dest, uint node_index) const {
   assert(dest.is_valid(),
          "Allocation buffer index out of bounds: %s", dest.get_type_str());
-  assert(_alloc_buffers[dest.type()] != NULL,
-         "Allocation buffer is NULL: %s", dest.get_type_str());
+  assert(_dest_data[dest.type()]._alloc_buffer != nullptr,
+         "Allocation buffer is null: %s", dest.get_type_str());
   return alloc_buffer(dest.type(), node_index);
 }
 
@@ -101,9 +91,9 @@ inline PLAB* G1PLABAllocator::alloc_buffer(region_type_t dest, uint node_index)
   if (dest == G1HeapRegionAttr::Young) {
     assert(node_index < alloc_buffers_length(dest),
            "Allocation buffer index out of bounds: %u, %u", dest, node_index);
-    return _alloc_buffers[dest][node_index];
+    return _dest_data[dest]._alloc_buffer[node_index];
   } else {
-    return _alloc_buffers[dest][0];
+    return _dest_data[dest]._alloc_buffer[0];
   }
 }
 
@@ -127,55 +117,10 @@ inline HeapWord* G1PLABAllocator::allocate(G1HeapRegionAttr dest,
                                            bool* refill_failed,
                                            uint node_index) {
   HeapWord* const obj = plab_allocate(dest, word_sz, node_index);
-  if (obj != NULL) {
+  if (obj != nullptr) {
     return obj;
   }
   return allocate_direct_or_new_plab(dest, word_sz, refill_failed, node_index);
 }
 
-inline bool G1PLABAllocator::needs_bot_update(G1HeapRegionAttr dest) const {
-  return dest.is_old();
-}
-
-inline void G1PLABAllocator::update_bot_for_direct_allocation(G1HeapRegionAttr attr, HeapWord* addr, size_t size) {
-  if (!needs_bot_update(attr)) {
-    return;
-  }
-
-  // Out of PLAB allocations in an old generation region. Update BOT.
-  HeapRegion* region = _g1h->heap_region_containing(addr);
-  region->update_bot_at(addr, size);
-}
-
-inline void G1PLABAllocator::update_bot_for_plab_allocation(G1HeapRegionAttr dest, size_t word_sz, uint node_index) {
-  assert(needs_bot_update(dest), "Wrong destination: %s", dest.get_type_str());
-  G1BotUpdatingPLAB* plab = static_cast<G1BotUpdatingPLAB*>(alloc_buffer(dest, node_index));
-  plab->update_bot(word_sz);
-}
-
-inline void G1BotUpdatingPLAB::set_buf(HeapWord* buf, size_t word_sz) {
-  PLAB::set_buf(buf, word_sz);
-  // Update the region and threshold to allow efficient BOT updates.
-  _region = G1CollectedHeap::heap()->heap_region_containing(buf);
-  _next_bot_threshold = _region->bot_threshold_for_addr(buf);
-}
-
-inline void G1BotUpdatingPLAB::update_bot(size_t word_sz) {
-  // The last object end is at _top, if it did not cross the
-  // threshold, there is nothing to do.
-  if (_top <= _next_bot_threshold) {
-    return;
-  }
-
-  HeapWord* obj_start = _top - word_sz;
-  assert(contains(obj_start),
-         "Object start outside PLAB. bottom: " PTR_FORMAT " object: " PTR_FORMAT,
-         p2i(_bottom), p2i(obj_start));
-  assert(obj_start <= _next_bot_threshold,
-         "Object start not below or at threshold. threshold: " PTR_FORMAT " object: " PTR_FORMAT,
-         p2i(_next_bot_threshold), p2i(obj_start));
-
-  _region->update_bot_crossing_threshold(&_next_bot_threshold, obj_start, _top);
-}
-
 #endif // SHARE_GC_G1_G1ALLOCATOR_INLINE_HPP
