@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,13 +25,12 @@
 
 package sun.security.ssl;
 
-import sun.security.action.GetPropertyAction;
-import sun.security.ssl.SSLExtension.ExtensionConsumer;
-import sun.security.ssl.SSLExtension.SSLExtensionSpec;
-import sun.security.ssl.SSLHandshake.HandshakeMessage;
-import sun.security.ssl.SupportedGroupsExtension.SupportedGroups;
-import sun.security.util.HexDumpEncoder;
-
+import java.io.IOException;
+import java.nio.ByteBuffer;
+import java.security.NoSuchAlgorithmException;
+import java.security.SecureRandom;
+import java.text.MessageFormat;
+import java.util.Locale;
 import javax.crypto.Cipher;
 import javax.crypto.KeyGenerator;
 import javax.crypto.SecretKey;
@@ -41,14 +40,12 @@ import javax.net.ssl.SSLSessionContext;
 
 import static sun.security.ssl.SSLExtension.CH_SESSION_TICKET;
 import static sun.security.ssl.SSLExtension.SH_SESSION_TICKET;
-import static sun.security.ssl.SignatureScheme.CERTIFICATE_SCOPE;
 
-import java.io.IOException;
-import java.nio.ByteBuffer;
-import java.security.NoSuchAlgorithmException;
-import java.security.SecureRandom;
-import java.text.MessageFormat;
-import java.util.Locale;
+import sun.security.action.GetPropertyAction;
+import sun.security.ssl.SSLExtension.ExtensionConsumer;
+import sun.security.ssl.SSLExtension.SSLExtensionSpec;
+import sun.security.ssl.SSLHandshake.HandshakeMessage;
+import sun.security.util.HexDumpEncoder;
 
 /**
  * SessionTicketExtension is an implementation of RFC 5077 with some internals
@@ -298,9 +295,10 @@ final class SessionTicketExtension {
             }
 
             MessageFormat messageFormat = new MessageFormat(
-                    "  \"ticket\" : '{'\n" +
-                            "{0}\n" +
-                            "  '}'",
+                    """
+                              "ticket" : '{'
+                            {0}
+                              '}'""",
                     Locale.ENGLISH);
             HexDumpEncoder hexEncoder = new HexDumpEncoder();
 
@@ -325,7 +323,7 @@ final class SessionTicketExtension {
     }
 
     private static final class T12CHSessionTicketProducer
-            extends SupportedGroups implements HandshakeProducer {
+            implements HandshakeProducer {
         T12CHSessionTicketProducer() {
         }
 
@@ -356,12 +354,11 @@ final class SessionTicketExtension {
                 return new byte[0];
             }
 
-            if (chc.localSupportedCertSignAlgs == null) {
-                chc.localSupportedCertSignAlgs =
+            if (chc.localSupportedSignAlgs == null) {
+                chc.localSupportedSignAlgs =
                         SignatureScheme.getSupportedAlgorithms(
                                 chc.sslConfig,
-                                chc.algorithmConstraints, chc.activeProtocols,
-                                CERTIFICATE_SCOPE);
+                                chc.algorithmConstraints, chc.activeProtocols);
             }
 
             return chc.resumingSession.getPskIdentity();
@@ -425,7 +422,7 @@ final class SessionTicketExtension {
 
 
     private static final class T12SHSessionTicketProducer
-            extends SupportedGroups implements HandshakeProducer {
+            implements HandshakeProducer {
         T12SHSessionTicketProducer() {
         }
 
