@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -101,7 +101,7 @@ class MultiExchange<T> implements Cancelable {
     );
 
     private final List<HeaderFilter> filters;
-    ResponseTimerEvent responseTimerEvent;
+    volatile ResponseTimerEvent responseTimerEvent;
     volatile boolean cancelled;
     AtomicReference<CancellationException> interrupted = new AtomicReference<>();
     final PushGroup<T> pushGroup;
@@ -169,9 +169,10 @@ class MultiExchange<T> implements Cancelable {
         this.responseHandler = responseHandler;
 
         if (pushPromiseHandler != null) {
+            Executor ensureExecutedAsync = this.executor::ensureExecutedAsync;
             Executor executor = acc == null
-                    ? this.executor.delegate()
-                    : new PrivilegedExecutor(this.executor.delegate(), acc);
+                    ? ensureExecutedAsync
+                    : new PrivilegedExecutor(ensureExecutedAsync, acc);
             this.pushGroup = new PushGroup<>(pushPromiseHandler, request, executor);
         } else {
             pushGroup = null;
@@ -210,12 +211,16 @@ class MultiExchange<T> implements Cancelable {
         return vers;
     }
 
-    private synchronized void setExchange(Exchange<T> exchange) {
-        if (this.exchange != null && exchange != this.exchange) {
-            this.exchange.released();
-            if (cancelled) exchange.cancel();
+    private void setExchange(Exchange<T> exchange) {
+        Exchange<T> previousExchange;
+        synchronized (this) {
+            previousExchange = this.exchange;
+            this.exchange = exchange;
         }
-        this.exchange = exchange;
+        if (previousExchange != null && exchange != previousExchange) {
+            previousExchange.released();
+        }
+        if (cancelled) exchange.cancel();
     }
 
     public Optional<Duration> remainingConnectTimeout() {
@@ -226,6 +231,7 @@ class MultiExchange<T> implements Cancelable {
     private void cancelTimer() {
         if (responseTimerEvent != null) {
             client.cancelTimer(responseTimerEvent);
+            responseTimerEvent = null;
         }
     }
 
@@ -421,6 +427,7 @@ class MultiExchange<T> implements Cancelable {
                             }
                             return completedFuture(response);
                         } else {
+                            cancelTimer();
                             this.response =
                                 new HttpResponseImpl<>(currentreq, response, this.response, null, exch);
                             Exchange<T> oldExch = exch;
