@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -49,6 +49,8 @@ public class Longs {
     @Param("500")
     private int size;
 
+    private long bound;
+    private long[] res;
     private String[] strings;
     private long[] longArraySmall;
     private long[] longArrayBig;
@@ -56,7 +58,9 @@ public class Longs {
     @Setup
     public void setup() {
         var random = ThreadLocalRandom.current();
+        bound = 20000L;
         strings = new String[size];
+        res = new long[size];
         longArraySmall = new long[size];
         longArrayBig = new long[size];
         for (int i = 0; i < size; i++) {
@@ -89,6 +93,22 @@ public class Longs {
         }
     }
 
+    /** Performs expand on small values */
+    @Benchmark
+    public void expand(Blackhole bh) {
+        for (long i : longArraySmall) {
+            bh.consume(Long.expand(i, 0xFF00F0F0F0000000L));
+        }
+    }
+
+    /** Performs compress on large values */
+    @Benchmark
+    public void compress(Blackhole bh) {
+        for (long i : longArrayBig) {
+            bh.consume(Long.compress(i, 0x000000000F0F0F1FL));
+        }
+    }
+
     /*
      * Have them public to avoid total unrolling
      */
@@ -104,4 +124,55 @@ public class Longs {
         }
         return x;
     }
+
+    @Benchmark
+    public void shiftRight(Blackhole bh) {
+        for (int i = 0; i < size; i++) {
+            bh.consume(longArrayBig[i] >> longArraySmall[i]);
+        }
+    }
+
+    @Benchmark
+    public void shiftURight(Blackhole bh) {
+        for (int i = 0; i < size; i++) {
+            bh.consume(longArrayBig[i] >>> longArraySmall[i]);
+        }
+    }
+
+    @Benchmark
+    public void shiftLeft(Blackhole bh) {
+        for (int i = 0; i < size; i++) {
+            bh.consume(longArrayBig[i] << longArraySmall[i]);
+        }
+    }
+
+    @Benchmark
+    public void compareUnsignedIndirect(Blackhole bh) {
+        for (int i = 0; i < size; i++) {
+            int r = (Long.compareUnsigned(longArraySmall[i], bound - 16) < 0) ? 1 : 0;
+            bh.consume(r);
+        }
+    }
+
+    @Benchmark
+    public void compareUnsignedDirect(Blackhole bh) {
+        for (int i = 0; i < size; i++) {
+            int r = Long.compareUnsigned(longArraySmall[i], bound - 16);
+            bh.consume(r);
+        }
+    }
+
+    @Benchmark
+    public void reverseBytes() {
+        for (int i = 0; i < size; i++) {
+            res[i] = Long.reverseBytes(longArraySmall[i]);
+        }
+    }
+
+    @Benchmark
+    public void reverse() {
+        for (int i = 0; i < size; i++) {
+            res[i] = Long.reverse(longArraySmall[i]);
+        }
+    }
 }
